#include <stdio.h>
#include <stdlib.h>

  /* Binarno potenciranje:
       Test za n > 0 i korektna inicijalizacija potencije za
       prvu ne-nul znamenku u n. Usteda mnozenja s 1.
  */

/****************************************************************************/

    /* Binarno potenciranje x^n. */

    /* Pretpostavka (bez provjere): n >= 0 i x != 0. */

    /* Rezultat se inicijalizira na prema prvoj (najnizoj) binarnoj
       znamenki od n koja jednaka 1. To u potpunosti izbjegava mnozenje
       s 1 i stedi (to) jedno mnozenje.
       Slozenost: O(lg n) mnozenja.
    */

double pow_bin(double x, int n)
{
    double pot, b = x;

        /* Prvo rijesimo specijalni slucaj n = 0. */

    if (n == 0) return 1.0;

        /* Potenciranje kvadriranjem i mnozenjem. */

        /* Prije petlje inicijaliziramo pot prema prvoj (najnizoj)
           ne-nula binarnoj znamenki od n. Takva znamenka (jedinica)
           mora postojati cim je n > 0. */

        /* Sve dok je (trenutni) n paran (zadnja binarna znamenka
           je jednaka 0), kvadriraj b i obrisi zadnju znamenku u n. */

        /* Za provjeru parnosti od n, umjesto n % 2 == 0,
           koristimo test zadnjeg bita u n, tj. (n & 1) == 0. */

    while ((n & 1) == 0) {

            /* Kvadriraj b. */

        b *= b;

            /* Obrisi zadnju binarnu znamenku u n, tj. podijeli n s 2.
               Brza realizacija je pomak za jedno mjesto udesno (>> 1). */

        n >>= 1;    /* n /= 2; */
    }

        /* Zadnja binarna znamenka (trenutnog) n je jednaka 1, tj.
           n je neparan. Inicijaliziraj pot na b. */

    pot = b;

        /* Obrisi zadnju binarnu znamenku u n, tj. podijeli n s 2.
           Brza realizacija je pomak za jedno mjesto udesno (>> 1). */

    n >>= 1;    /* n /= 2; */


        /* Petlja za preostale binarne znamenke od n. */

    while (n > 0) {

            /* Kvadriraj b. */

        b *= b;

            /* Ako je zadnja znamenka od n jednaka 1 (n neparan),
               onda akumuliraj produkt u pot (pomnozi pot s b). */
            /* Za provjeru neparnosti od n, umjesto n % 2 == 1,
               koristimo test zadnjeg bita u n, tj. (n & 1) != 0.
               Dovoljno je samo gledati n & 1 (kao logicki izraz). */

        if ((n & 1) != 0) pot *= b;    /* if (n % 2 == 1) pot *= b; */

            /* Obrisi zadnju binarnu znamenku u n, tj. podijeli n s 2.
               Brza realizacija je pomak za jedno mjesto udesno (>> 1). */

        n >>= 1;    /* n /= 2; */
    }

    return pot;
}

    /* Cjelobrojno potenciranje x^n.
       Koristi funkciju za x^abs(n). */

double int_pow(double x, int n)
{
    double pot;
    int neg;

        /* Provjera x = 0. */
    if (x == 0.0) return 0.0;

        /* Zapamti predznak od n. */
    neg = n < 0;
    n = abs(n);

        /* Potenciranje kvadriranjem i mnozenjem
           (binarno potenciranje). */
    pot = pow_bin(x, n);

    if (neg) pot = 1.0 / pot;
    return pot;
}

int main(void)
{
    double x;
    int n;

    x = 2.0;
    for (n = 0; n <= 16; ++n)
        printf(" Potencija %3.1f na %2d = %7.1f\n",
                 x, n, int_pow(x, n) );

    n = 1000000000;     /* n = 10^9 */
    x = 1.000000001;    /* x = 1 + 1.0/n */

    printf(" Potencija %11.9f na %d = %11.9f\n",
           x, n, int_pow(x, n) );

    return 0;
}
