#include <stdio.h>
#include <stdlib.h>
#include <time.h>

  /* Autor i ideja: Vedran Sego, 20. Dec 2012. (nastupno predavanje).

     Usporedba sekvencijalnog i binarnog trazenja na nizu (polju) "niz"
     od 1000000 slucajnih int-ova. Trazenje se radi za 10000 elemenata,
     spremljenih u nizu "trazi", od kojih se priblizno pola nalazi u nizu.

     Za promjenu duljine niza i broja trazenih elemenata, treba promijeniti
     dimenzije u deklaraciji polja "niz" i "trazi", na samom pocetku
     glavnog programa.

     Program ima 3 dijela:
       1. Sekvencijalno trazenje u nesortiranom nizu,
       2. Sortiranje nizu (qsort iz stdlib),
       3. Binarno trazenje u sortiranom nizu.
     Za svaki dio ispisuje se trajanje.
  */


    /* Funkcija za generiranje slucajnih int-ova u skoro punom rasponu.
       Standardni rand( ) daje brojeve od 0 do RAND_MAX = 32767 = 2^15 - 1.
       Zato se int "gradi" iz dva takva broja, uz pomak prvog za 15 bitova. */

int randi(void) {
    return (rand() << 15) + rand();
}

    /* Funkcija za usporedbu intova - treba za qsort. */

int cmp(const void *p1, const void *p2) {
    return *(int *)p1 - *(int *)p2;
}

    /* Funkcija za sekvencijalno trazenje u nizu. */

int sekv(int niz[], int n, int el)
{
    int i;
    for (i = 0; i < n; i++)
        if (niz[i] == el) return i;
    return -1;
}

    /* Funkcija za binarno trazenje (u sortiranom nizu). */

int bin(int niz[], int n, int el)
{
    int l = 0, d = n-1;
    while (l <= d) {
        int s = (l+d)/2;
        if (niz[s] == el) return s;
        if (el < niz[s]) d = s-1; else l = s+1;
    }
    return -1;
}

    /* Deklaracija globalnih polja "niz" i "trazi". */

int niz[1000000], trazi[10000];


    /* Glavni program. */

int main(void)
{
    int i;
    int n = sizeof(niz) / sizeof(int);
    int m = sizeof(trazi) / sizeof(int);
    double start, end;
    unsigned cnt_sekv = 0, cnt_bin = 0;

        /* Inicijalizacija sjemena (seed) za generator slucajnih brojeva. */
/* Izbaceno, zato da se moze ponoviti eksperiment s ISTIM brojevima.
    srand((unsigned)time(NULL));
*/

        /* Inicijalizacija elemenata u polju niz (na slucajne brojeve). */
    for (i = 0; i < n; i++)
        niz[i] = randi();

        /* Inicijalizacija elemenata u polju trazi, prema sljedecem
           slucajnom broju: ako je on paran, uzimamo element iz niz
           (na slucajnom indeksu) - zato da ga nadjemo. U protivnom,
           uzimamo sljedeci slucajni broj (moze/ne mora biti u niz). */
    for (i = 0; i < m; i++)
        trazi[i] = (randi() % 2 ? niz[randi() % n] : randi());

        /* 1. Sekvencijalno trazenje u nesortiranom nizu. */

    start = (double)clock() / CLOCKS_PER_SEC;
    for (i = 0; i < m; i++)
        if (sekv(niz, n, trazi[i]) >= 0) ++cnt_sekv;
    end = (double)clock() / CLOCKS_PER_SEC;
    printf(" Sekvencijalno: %g sekundi (%u nadjenih)\n",
           end - start, cnt_sekv);

        /* 2. Sortiranje nizu (qsort iz stdlib). */

    start = (double)clock() / CLOCKS_PER_SEC;
    qsort(niz, n, sizeof(int), cmp);
    end = (double)clock() / CLOCKS_PER_SEC;
    printf(" Vrijeme sortiranja: %g sekundi\n", end - start);

        /* 3. Binarno trazenje u sortiranom nizu. */

    start = (double)clock() / CLOCKS_PER_SEC;
    for (i = 0; i < m; i++)
        if (bin(niz, n, trazi[i]) >= 0) ++cnt_bin;
    end = (double)clock() / CLOCKS_PER_SEC;
    printf(" Binarno: %g sekundi (%u nadjenih)\n", end - start, cnt_bin);

    return 0;
}
