#include <stdio.h>

/****************************************************************************/

  /* Insertion sort algoritam (sortiranje umetanjem).
     Ubrzana varijanta - prvo nadji najmanjeg i dovedi ga na pocetak.
     Sortirani dio je spijeda. Binarno trazenje, onda pomaci.
     Izbjegava nepotrebna kopiranja ako je novi element najveci.
  */

/****************************************************************************/

  /* Uzlazno sortiranje (sortiranim) umetanjem.
     Sortirani dio je sprijeda.
     Prvo dovodimo najmanji na pocetak.
     Umjesto sekvencijalnog, koristi binarno trazenje.
     Ako je sljedeci element veci, ne radi nista (nema kopiranja). */

void insertion_sort(int x[], int n)
{
    int i, j, l, d, m, temp;

        /* Prvo nadji najmanjeg i dovedi ga na pocetak. */
    i = 0;
    temp = x[0];
    for (j = 1; j < n; ++j)
        if (x[j] < temp) {
            i = j;
            temp = x[j];
        }
    if (i != 0) {
        x[i] = x[0];
        x[0] = temp;
    }

        /* Sljedeci element x[1] je veci od najmanjeg x[0],
           pa sortiranje umetanjem pocinjemo od x[2]. */
    for (i = 2; i < n; ++i) {

            /* Ako je x[i] >= x[i - 1], onda nemamo sto za raditi,
               jer je i polje x[0], ..., x[i] uzlazno sortirano,
               tj. x[i] je vec na korektnom mjestu. */

        if (x[i] < x[i - 1]) {
            temp = x[i];

                /* Zato sto je x[0] najmanji, nije potreban test
                   temp >= x[0], jer to sigurno vrijedi. */

                /* Binarno trazenje u x[0], ..., x[i - 1].
                   Trazimo indeks d takav da je x[d - 1] <= temp < x[d].
                   Uz inicijalizaciju l = 0, d = i - 1, uvijek vrijedi
                   x[l] <= temp < x[d]. Stajemo kad je l + 1 = d. */

            l = 0;
            d = i - 1;
            while (l + 1 < d) {
                m = (l + d) / 2;
                if (temp < x[m])
                    d = m;
                else    /* x[m] <= temp. */
                    l = m;
            }

                /* Pomak desnog dijela sortiranog polja za jedno mjesto udesno
                   x[d], ..., x[i - 1] --> x[d + 1], ..., x[i]. */
            for (j = i; j > d; --j)
                x[j] = x[j - 1];

            x[j] = temp;    /* Moze i x[d] = temp. */

        }    /* Kraj if (x[i] < x[i - 1]). */

    }    /* Kraj for po i. */

    return;
}

/****************************************************************************/

int main(void)
{
    int i, n;
    int x[] = {42, 12, 55, 94, 18, 44, 67};

    n = 7;
    insertion_sort(x, n);

    printf("\n Sortirano polje x:\n");
    for (i = 0; i < n; ++i) {
        printf(" x[%d] = %d\n", i, x[i]);
    }

    return 0;
}
